// This program reads AIN0 every second, sends the calculated temperature
// to a display and debug information via the serial port.

#include <reg167.h>
#include <stdio.h>
#include <math.h>
#include <intrins.h>
#include <string.h>

sbit P310 = P3^10;     //port 3 register/bit 10
sbit DP310 = DP3^10;   //DP3-port 3 direction control register

bit secflag;

float idata temp,idata k,idata r_temp,idata lnr;
float const a=1.0271091e-3, b=2.3942380e-4,
      	    c=1.5517327e-7, r_ref=10000;
// Steinhart-Hart thermistor coefficients , reference resistor

unsigned char idata buffer[7];

/******************************************************************/
void tickerint (void) interrupt 0x20 
{	secflag=1;
}
/******************************************************************/

void main(void)
{
//Initialisation

// Serial Port

   P310=1;         // set Port 3.10 output latch TxD 
   DP310=1;        // set Port 3.10 direction control TxD out      
   S0TIC=0x80;     // set transmit interrupt flag
   S0BG= 0x33;     // set baudrate 9600 @ 16 MC CPU clock
   S0CON=0x8011;   // set serial mode(1000 0000 0001 0001),
            		   // enable rec.,8bit data async,1stop bit

// Timer0

T01CON=0x0047;     //CAPCOM Timer 0&1 control register
	           // x 0 xx 0 000   x 1 xx 0 111
	           // timer mode run enable, prescaler 1024

T0= -15625;        //Timer 0 register
T0REL= -15625;     //timer0 reload register ticker 1 s 
T0IC=0x44;         //Timer 0 interrupt enable ILVL=1; GLVL=0;
		   
//AD converter

ADCON=0x0000;

puts("OK\n");

IEN=1;

while(1)
{
	   
while (!secflag) _idle_();
secflag=0;



ADST=1;        //ADSTART
while (ADBSY);
    
k=ADDAT & 0x03ff;  //ADDAT-A/D converter result register, 10 bits mask
r_temp=k*r_ref/(1024-k); // calculation of temperature
lnr=log(r_temp);         // with the Steinhart Hart equation 
 
temp =1/(a + b * lnr + c * lnr * lnr * lnr) - 273.15;
printf("%4.0f    %6.0f    %6.2f [C]\n",k,r_temp,temp);

sprintf(buffer,"%6.2f",temp); 
xmemcpy(0x20000,buffer,6);
}
}

